
module VGA_TEST(
	input wire 		clk,		//input clock, set according to frame rate: 40MHz
	//input wire 		clr,		//asynchronous reset
	output wire 		VGA_HS_O,		//horizontal sync out
	output wire 		VGA_VS_O,		//vertical sync out
	output reg [3:0] 	VGA_R,		//VGA_R vga output
	output reg [3:0] 	VGA_G, 		//VGA_G vga output
	output reg [3:0] 	VGA_B		//VGA_B vga output
	//output wire 		lcd_clk,	// serdes clock
	//output reg 		blank,		// data enable
	//output  		sync		// sync enable	
	);


// video structure constants 640x480 @ 60Hz (25MHz)
//parameter hpixels = 800;// horizontal pixels per line
//parameter vlines = 521; // vertical lines per frame
//parameter hpulse = 96; 	// VGA_HS_O pulse length
//parameter vpulse = 2; 	// VGA_VS_O pulse length
//parameter hbp = 144; 	// end of horizontal back porch
//parameter hfp = 784; 	// beginning of horizontal front porch
//parameter vbp = 31; 		// end of vertical back porch
//parameter vfp = 511; 	// beginning of vertical front porch
//parameter pxlcnt = 80;	// pixl count for 8 color bar
// active horizontal video is therefore: 784 - 144 = 640
// active vertical video is therefore: 511 - 31 = 480
 


// video structure constants 800x600 @ 56Hz (38MHz)
//parameter hpixels = 1088;// horizontal pixels per line
//parameter vlines = 615; // vertical lines per frame
//parameter hpulse = 128; 	// VGA_HS_O pulse length
//parameter vpulse = 4; 	// VGA_VS_O pulse length
//parameter hbp = 256; 	// end of horizontal back porch
//parameter hfp = 1056; 	// beginning of horizontal front porch
//parameter vbp = 14; 		// end of vertical back porch
//parameter vfp = 614; 	// beginning of vertical front porch
//parameter pxlcnt = 100;	// pixl count for 8 color bar
// active horizontal video is therefore: 1056 - 256 = 800
// active vertical video is therefore: 614 - 14 = 600


// video structure constants 800x600 @ 60Hz (40MHz)
parameter hpixels = 1056;	// horizontal pixels per line
parameter vlines = 624; 	// vertical lines per frame
parameter hpulse = 128; 	// VGA_HS_O pulse length
parameter vpulse = 4; 		// VGA_VS_O pulse length
parameter hbp = 216; 		// end of horizontal back porch
parameter hfp = 1016; 		// beginning of horizontal front porch
parameter vbp = 23; 		// end of vertical back porch
parameter vfp = 623; 		// beginning of vertical front porch
parameter pxlcnt = 100;		// pixl count for 8 color bar
// active horizontal video is therefore: 1016 - 216 = 800
// active vertical video is therefore: 623 - 23 = 600


// registers for storing the horizontal & vertical counters
reg [10:0] hc;
reg [10:0] vc;

// Horizontal & vertical counters --
// this is how we keep track of where we are on the screen.
// ------------------------
// Sequential "always block", which is a block that is
// only triggeVGA_R on signal transitions or "edges".
// posedge = rising edge  &  negedge = falling edge
// Assignment statements can only be used on type "reg" and need to be of the "non-blocking" type: <=
always @(posedge clk) begin
	// reset condition
//	if (clr == 0) 	begin
//		hc <= 0;
//		vc <= 0;
//	end
//	else 
//	begin
		// keep counting until the end of the line
		if (hc < hpixels - 1)
			hc <= hc + 1;
		else
		// When we hit the end of the line, reset the horizontal
		// counter and increment the vertical counter.
		// If vertical counter is at the end of the frame, then
		// reset that one too.
		begin
			hc <= 0;
			if (vc < vlines - 1)
				vc <= vc + 1;
			else
				vc <= 0;
		end
		
//	end
end

// generate sync pulses (active low)
// ----------------
// "assign" statements are a quick way to
// give values to variables of type: wire
assign VGA_HS_O = (hc < hpulse) ? 0:1;
assign VGA_VS_O = (vc < vpulse) ? 0:1;

// display 100% saturation colorbars
// ------------------------
// Combinational "always block", which is a block that is
// triggeVGA_R when anything in the "sensitivity list" changes.
// The asterisk implies that everything that is capable of triggering the block
// is automatically included in the sensitivty list.  In this case, it would be
// equivalent to the following: always @(hc, vc)
// Assignment statements can only be used on type "reg" and should be of the "blocking" type: =
always @(*)
begin
	// first check if we're within vertical active video range
	if (vc >= vbp && vc < vfp)
	begin
		// now display different colors every 100 pixels
		// while we're within the active horizontal range
		// -----------------
		// display white bar
		if (hc >= hbp && hc < (hbp+pxlcnt))
		begin
			VGA_R   = 4'b1111;
			VGA_G = 4'b1111;
			VGA_B  = 4'b1111;
			//blank = 1'b1;
		end
		// display yellow bar
		else if (hc >= (hbp+pxlcnt) && hc < (hbp+ 2*pxlcnt))
		begin
			VGA_R   = 4'b1111;
			VGA_G = 4'b1111;
			VGA_B  = 4'b0000;
			//blank = 1'b1;
		end
		// display cyan bar
		else if (hc >= (hbp+ 2*pxlcnt) && hc < (hbp+ 3*pxlcnt))
		begin
			VGA_R   = 4'b0000;
			VGA_G = 4'b1111;
			VGA_B  = 4'b1111;
			//blank = 1'b1;
		end
		// display VGA_G bar
		else if (hc >= (hbp+ 3*pxlcnt) && hc < (hbp+ 4*pxlcnt))
		begin
			VGA_R   = 4'b0000;
			VGA_G = 4'b1111;
			VGA_B  = 4'b0000;
			//blank = 1'b1;
		end
		// display magenta bar
		else if (hc >= (hbp+ 4*pxlcnt) && hc < (hbp+ 5*pxlcnt))
		begin
			VGA_R   = 4'b1111;
			VGA_G = 4'b0000;
			VGA_B  = 4'b1111;
			//blank = 1'b1;
		end
		// display VGA_R bar
		else if (hc >= (hbp+ 5*pxlcnt) && hc < (hbp+ 6*pxlcnt))
		begin
			VGA_R   = 4'b1111;
			VGA_G = 4'b0000;
			VGA_B  = 4'b0000;
			//blank = 1'b1;
		end
		// display VGA_B bar
		else if (hc >= (hbp+ 6*pxlcnt) && hc < (hbp+ 7*pxlcnt))
		begin
			VGA_R   = 4'b0000;
			VGA_G = 4'b0000;
			VGA_B  = 4'b1111;
			//blank = 1'b1;
		end
		// display black bar
		else if (hc >= (hbp+ 7*pxlcnt) && hc < (hbp+ 8*pxlcnt))
		begin
			VGA_R   = 4'b0000;
			VGA_G = 4'b0000;
			VGA_B  = 4'b0000;
			//blank = 1'b1;
		end
		// we're outside active horizontal range so display black
		else
		begin
			VGA_R   = 0;
			VGA_G = 0;
			VGA_B  = 0;
			//blank = 1'b0;
		end
	end
	// we're outside active vertical range so display black
	else
	begin
		VGA_R   = 0;
		VGA_G = 0;
		VGA_B  = 0;
		//blank = 1'b0;
	end
end

//assign 	sync 	= 1'b0;
//assign 	lcd_clk	= clr? clk : 1'b0;
endmodule